@@ -19,6 +19,7 @@
 #include "gscreen.h"
 #include "main.h"
 #include "gmain.h"
+#include "gcmd.h"
 #include "gcorba.h"
 
 PortableServer_POA poa;
@@ -124,7 +125,12 @@ do_window_close(GtkWidget *widget, gpointer _servant)
 	CORBA_exception_init(&ev);
 	objid = PortableServer_POA_servant_to_id (servant->poa, _servant, &ev);
 	PortableServer_POA_deactivate_object (servant->poa, objid, &ev);
-	CORBA_free(objid);
+#if 1
+#warning Remove this ifdef when the new ORBit is released.
+	g_warning ("Developer Warning: when a new ORBIt is released, remove this code\n");
+#else
+	CORBA_free (objid);
+#endif
 	POA_GNOME_FileManagerWindow__fini(_servant, &ev);
 	
 	g_free(servant);
@@ -134,7 +140,7 @@ do_window_close(GtkWidget *widget, gpointer _servant)
 static void
 impl_GNOME_FileManagerWindow_close (impl_POA_GNOME_FileManagerWindow * servant, CORBA_Environment * ev)
 {
-	gtk_widget_destroy(servant->mywin->xwindow);
+	gnome_close_panel (GTK_WIDGET (servant->mywin->widget.wdata), servant->mywin);
 }
 
 GNOME_FileManagerWindow
@@ -152,7 +158,7 @@ impl_GNOME_FileManagerFactory_create_window (impl_POA_GNOME_FileManagerFactory *
 	newservant->mywin = new_panel_at ((dir && *dir) ? dir : home_dir);
 	
 	gtk_signal_connect(GTK_OBJECT(newservant->mywin->xwindow),
-			   "destroy", impl_GNOME_FileManagerWindow_close,
+			   "destroy", do_window_close,
 			   newservant);
 	
 	POA_GNOME_FileManagerWindow__init((PortableServer_Servant) newservant, ev);
@@ -235,7 +241,7 @@ corba_init (void)
 
 	orb = gnome_CORBA_ORB ();
 
-	poa = CORBA_ORB_resolve_initial_references (orb, "RootPOA", &ev);
+	poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references (orb, "RootPOA", &ev);
 	if (ev._major != CORBA_NO_EXCEPTION){
 		printf ("Can not resolve initial reference to RootPOA");
 		return;
