@@ -122,45 +122,78 @@ static int click_current_y;
 static int click_dragging;
 
 
-static DesktopIconInfo *desktop_icon_info_new (char *filename, char *url,
-					       char *caption,
-					       int user_pos, int auto_pos,
+static DesktopIconInfo *desktop_icon_info_new (char *filename, char *url, char *caption,
 					       int xpos, int ypos);
 
 
-/* Looks for a free slot in the layout_slots array and returns the coordinates that coorespond to
- * it.  "Free" means it either has zero icons in it, or it has the minimum number of icons of all
- * the slots.
+/* Convenience function to figure out the slot corresponding to an (x, y) position */
+static void
+get_slot_from_pos (int x, int y, int *u, int *v)
+{
+	*u = (x + DESKTOP_SNAP_X / 2) / DESKTOP_SNAP_X;
+	*v = (y + DESKTOP_SNAP_Y / 2) / DESKTOP_SNAP_Y;
+}
+
+/* Looks for a free slot in the layout_slots array and returns the coordinates
+ * that coorespond to it.  "Free" means it either has zero icons in it, or it
+ * has the minimum number of icons of all the slots.  Returns the number of
+ * icons in the sought spot (ideally 0).
  */
+static int
+auto_pos (int start_slot, int end_slot, int *slot)
+{
+	int min, min_slot;
+	int i;
+	int val;
+
+	min = layout_slots[start_slot].num_icons;
+	min_slot = start_slot;
+
+	for (i = start_slot; i < end_slot; i++) {
+		val = layout_slots[i].num_icons;
+
+		if (val < min || val == 0) {
+			min = val;
+			min_slot = i;
+			if (val == 0)
+				break;
+		}
+	}
+
+	*slot = min_slot;
+	return min;
+}
+
+/* Looks for free space in the icon grid, scanning it in column-wise order */
 static void
 get_icon_auto_pos (int *x, int *y)
 {
-	int min, min_x, min_y;
+	int start, end;
 	int u, v;
-	int val;
+	int val1, val2;
+	int slot1, slot2;
+	int slot;
 
-	min = l_slots (0, 0).num_icons;
-	min_x = min_y = 0;
+	get_slot_from_pos (*x, *y, &u, &v);
+	start = u * layout_rows + v;
+	end = layout_rows * layout_cols;
 
-	for (u = 0; u < layout_cols; u++)
-		for (v = 0; v < layout_rows; v++) {
-			val = l_slots (u, v).num_icons;
+	/* Look forwards until the end of the grid.  If we could not find an
+	 * empty spot, find the second best.
+	 */
 
-			if (val == 0) {
-				/* Optimization: if it is zero, return immediately */
+	val1 = auto_pos (start, end, &slot1);
 
-				*x = u * DESKTOP_SNAP_X;
-				*y = v * DESKTOP_SNAP_Y;
-				return;
-			} else if (val < min) {
-				min = val;
-				min_x = u;
-				min_y = v;
-			}
-		}
+	if (val1 == 0)
+		slot = slot1;
+	else {
+		val2 = auto_pos (0, start, &slot2);
+		if (val2 < val1)
+			slot = slot2;
+	}
 
-	*x = min_x * DESKTOP_SNAP_X;
-	*y = min_y * DESKTOP_SNAP_Y;
+	*x = (slot / layout_rows) * DESKTOP_SNAP_X;
+	*y = (slot % layout_rows) * DESKTOP_SNAP_Y;
 }
 
 /* Snaps the specified position to the icon grid.  It looks for the closest free spot on the grid,
@@ -211,28 +244,21 @@ remove_from_slot (DesktopIconInfo *dii)
 
 	layout_slots[dii->slot].num_icons--;
 	layout_slots[dii->slot].icons = g_list_remove (layout_slots[dii->slot].icons, dii);
+
+	dii->slot = -1;
+	dii->x = 0;
+	dii->y = 0;
 }
 
-/* Places a desktop icon.  If auto_pos is true, then the function will look for
- * a place to position the icon automatically, else it will use the specified
- * coordinates, snapped to the grid if the global desktop_snap_icons flag is
- * set.
- */
+/* Places a desktop icon on the specified position */
 static void
-desktop_icon_info_place (DesktopIconInfo *dii, int user_pos, int auto_pos, int xpos, int ypos)
+desktop_icon_info_place (DesktopIconInfo *dii, int xpos, int ypos)
 {
 	int u, v;
 	char *filename;
 
 	remove_from_slot (dii);
 
-	if (!user_pos && auto_pos) {
-		if (desktop_snap_icons)
-			get_icon_snap_pos (&xpos, &ypos);
-		else
-			get_icon_auto_pos (&xpos, &ypos);
-	}
-
 	if (xpos < 0)
 		xpos = 0;
 	else if (xpos > layout_screen_width)
@@ -245,8 +271,7 @@ desktop_icon_info_place (DesktopIconInfo *dii, int user_pos, int auto_pos, int x
 
 	/* Increase the number of icons in the corresponding slot */
 
-	u = (xpos + DESKTOP_SNAP_X / 2) / DESKTOP_SNAP_X;
-	v = (ypos + DESKTOP_SNAP_X / 2) / DESKTOP_SNAP_Y;
+	get_slot_from_pos (xpos, ypos, &u, &v);
 
 	dii->slot = u * layout_rows + v;
 	layout_slots[dii->slot].num_icons++;
@@ -326,8 +351,7 @@ typedef struct {
 	char *caption;
 } file_and_url_t;
 
-/*
- * Reloads the desktop icons efficiently.  If there are "new" files for which no
+/* Reloads the desktop icons efficiently.  If there are "new" files for which no
  * icons have been created, then icons for them will be created started at the
  * specified position if user_pos is TRUE.  If it is FALSE, the icons will be
  * auto-placed.
@@ -343,6 +367,7 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 	GSList *need_position_list, *sl;
 	GList *all_icons, *l;
 	char *desktop_url, *caption;
+	int orig_xpos, orig_ypos;
 
 	dir = mc_opendir (desktop_directory);
 	if (!dir) {
@@ -404,7 +429,7 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 		gnome_metadata_get (full_name, "icon-caption", &size, &caption);
 		
 		if (have_pos) {
-			dii = desktop_icon_info_new (dirent->d_name, desktop_url, caption, FALSE, FALSE, x, y);
+			dii = desktop_icon_info_new (dirent->d_name, desktop_url, caption, x, y);
 			gtk_widget_show (dii->dicon);
 
 			g_free (full_name);
@@ -452,10 +477,31 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 
 	need_position_list = g_slist_reverse (need_position_list);
 
+	orig_xpos = orig_ypos = 0;
+
 	for (sl = need_position_list; sl; sl = sl->next) {
 		file_and_url_t *fau = sl->data;
 
-		dii = desktop_icon_info_new (fau->filename, fau->url, fau->caption, user_pos, TRUE, xpos, ypos);
+		if (user_pos && sl == need_position_list) {
+			/* If we are on the first icon, place it "by hand".
+			 * Else, use automatic placement based on the position
+			 * of the first icon of the series.
+			 */
+			if (desktop_auto_placement) {
+				xpos = ypos = 0;
+				get_icon_auto_pos (&xpos, &ypos);
+			} else if (desktop_snap_icons)
+				get_icon_snap_pos (&xpos, &ypos);
+
+			orig_xpos = xpos;
+			orig_ypos = ypos;
+		} else {
+			xpos = orig_xpos;
+			ypos = orig_ypos;
+			get_icon_auto_pos (&xpos, &ypos);
+		}
+
+		dii = desktop_icon_info_new (fau->filename, fau->url, fau->caption, xpos, ypos);
 		gtk_widget_show (dii->dicon);
 
 		if (fau->url)
@@ -467,13 +513,32 @@ reload_desktop_icons (int user_pos, int xpos, int ypos)
 	}
 
 	g_slist_free (need_position_list);
-
 	gnome_metadata_unlock ();
 
 	/* Flush events to make the icons paint themselves */
 	x_flush_events ();
 }
 
+/* Perform automatic arrangement of the desktop icons */
+static void
+arrange_desktop_icons (void)
+{
+	GList *icons, *l;
+	int xpos, ypos;
+
+	icons = g_list_reverse (get_all_icons ());
+
+	for (l = icons; l; l = l->next)
+		remove_from_slot (l->data);
+
+	for (l = icons; l; l = l->next) {
+		get_icon_auto_pos (&xpos, &ypos);
+		desktop_icon_info_place (l->data, xpos, ypos);
+	}
+
+	g_list_free (icons);
+}
+
 /* Unselects all the desktop icons except the one in exclude */
 static void
 unselect_all (DesktopIconInfo *exclude)
@@ -1278,21 +1343,22 @@ drop_desktop_icons (GdkDragContext *context, GtkSelectionData *data, int x, int
 				sel_icons = g_slist_prepend (sel_icons, l->data);
 		}
 
-	/* Move the icons */
+	/* Move the icons.  FIXME: handle auto-placement by reinserting the
+	 * icons in the proper place.
+	 */
 
-	for (sl = sel_icons; sl; sl = sl->next) {
-		dii = sl->data;
-		desktop_icon_info_place (dii,
-					 !desktop_auto_placement,
-					 desktop_auto_placement,
-					 dii->x + dx, dii->y + dy);
-	}
+	if (!desktop_auto_placement)
+		for (sl = sel_icons; sl; sl = sl->next) {
+			dii = sl->data;
+			desktop_icon_info_place (dii, dii->x + dx, dii->y + dy);
+		}
 
 	/* Clean up */
 
 	g_slist_free (sel_icons);
 }
 
+/* Handler for drag_data_received for desktop icons */
 static void
 icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 			 GtkSelectionData *data, guint info, guint time, gpointer user_data)
@@ -1313,7 +1379,7 @@ icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gin
 			return; /* eeeek */
 
 		if (gdnd_perform_drop (context, data, fe, full_name))
-			reload_desktop_icons (TRUE, x, y);
+			reload_desktop_icons (FALSE, 0, 0);
 
 		file_entry_free (fe);
 	}
@@ -1341,7 +1407,7 @@ setup_icon_dnd_dest (DesktopIconInfo *dii)
  * desktop directory.  It does not show the icon.
  */
 static DesktopIconInfo *
-desktop_icon_info_new (char *filename, char *url, char *caption, int user_pos, int auto_pos, int xpos, int ypos)
+desktop_icon_info_new (char *filename, char *url, char *caption, int xpos, int ypos)
 {
 	DesktopIconInfo *dii;
 	file_entry *fe;
@@ -1427,7 +1493,7 @@ desktop_icon_info_new (char *filename, char *url, char *caption, int user_pos, i
 
 	/* Place the icon and append it to the list */
 
-	desktop_icon_info_place (dii, user_pos, auto_pos, xpos, ypos);
+	desktop_icon_info_place (dii, xpos, ypos);
 	return dii;
 }
 
@@ -1519,7 +1585,10 @@ create_desktop_dir (void)
 {
 	char *home_link_name;
 
-	desktop_directory = g_concat_dir_and_file (gnome_user_home_dir, DESKTOP_DIR_NAME);
+	if (getenv ("GNOME_DESKTOP_DIR") != NULL)
+		desktop_directory = g_strdup (getenv ("GNOME_DESKTOP_DIR"));
+	else
+		desktop_directory = g_concat_dir_and_file (gnome_user_home_dir, DESKTOP_DIR_NAME);
 
 	if (!g_file_exists (desktop_directory)) {
 		/* Create the directory */
@@ -1822,11 +1891,36 @@ find_click_proxy_window (void)
 	return proxy_gdk_window;
 }
 
+/* Callback for arranging the icons on the desktop */
+static void
+handle_arrange_icons (GtkWidget *widget, gpointer data)
+{
+	arrange_desktop_icons ();
+}
+
+/* Callback for rescanning the desktop directory */
+static void
+handle_rescan_desktop (GtkWidget *widget, gpointer data)
+{
+	reload_desktop_icons (FALSE, 0, 0);
+}
+
+/* The popup menu for the desktop */
+static GnomeUIInfo desktop_popup_items[] = {
+	GNOMEUIINFO_ITEM_NONE (N_("Arrange Icons"), NULL, handle_arrange_icons),
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan Desktop"), NULL, handle_rescan_desktop),
+	GNOMEUIINFO_END
+};
+
 /* Executes the popup menu for the desktop */
 static void
 desktop_popup (GdkEventButton *event)
 {
-	printf ("FIXME: display desktop popup menu\n");
+	GtkWidget *popup;
+
+	popup = gnome_popup_menu_new (desktop_popup_items);
+	gnome_popup_menu_do_popup_modal (popup, NULL, NULL, event, NULL);
+	gtk_widget_destroy (popup);
 }
 
 /* Draws the rubberband rectangle for selecting icons on the desktop */
