@@ -79,7 +79,7 @@
 #    	 endif
 #    endif
  
-#    include "regex.h"
+#    include <regex.h>
 
 #endif
 
@@ -110,7 +110,6 @@
 #else
 
 #    include "src/main.h"		/* for char *shell */
-#    include "src/mad.h"
 #    include "src/dlg.h"
 #    include "src/widget.h"
 #    include "src/color.h"
@@ -123,7 +122,7 @@
 #    include "src/win.h"
 #    include "vfs/vfs.h"
 #    include "src/menu.h"
-#    include "src/regex.h"
+#    include <regex.h>
 #    define WANT_WIDGETS
      
 #    define WIDGET_COMMAND (WIDGET_USER + 10)
