@@ -23,7 +23,7 @@
 
 #include <config.h>
 #include <stdio.h>
-#if defined(__os2__)            /* OS/2 need io.h! .ado */
+#if defined(NEEDS_IO_H)            /* OS/2 need io.h! .ado */
 #    include <io.h>
 #endif
 #include <sys/types.h>
@@ -76,7 +76,7 @@
 #include "user.h"		/* expand_format */
 #include "../vfs/vfs.h"
 
-/* "$Id: util.c,v 1.17 1999/02/03 23:19:43 norbert Exp $" */
+/* "$Id: util.c,v 1.18 1999/02/13 20:12:52 unammx Exp $" */
 
 char app_text [] = "Midnight-Commander";
 
@@ -659,8 +659,12 @@ char *x_basename (char *s)
 void my_putenv (char *name, char *data)
 {
     char *full;
+    char *len = strlen (name) + strlen (data) + 2;
 
-    full = g_strdup_printf ("%s=%s", name, data);
+    full = malloc (len);
+    strcpy (full, name);
+    strcat (full, "=");
+    strcat (full, data);
     putenv (full);
 
     /* WARNING: NEVER FREE THE full VARIABLE!!!!!!!!!!!!!!!!!!!!!!!! */
